@@ -19,9 +19,10 @@ from .eou import EOU
 from .metrics import cascading_metrics_collector
 from .denoise import Denoise
 from .utils import UserState, AgentState
+import uuid
+from .utterance_handle import UtteranceHandle
 import logging
-import wave
-from .background_audio import BackgroundAudio, BackgroundAudioConfig
+from .background_audio import BackgroundAudioHandler, BackgroundAudioHandlerConfig
 
 logger = logging.getLogger(__name__)
 
@@ -44,8 +45,6 @@ class ConversationFlow(EventEmitter[Literal["transcription"]], ABC):
         self.agent = agent
         self.denoise = denoise
         self._stt_started = False
-        self.background_audio: BackgroundAudioConfig | None = None
-        self._background_audio_player: BackgroundAudio | None = None
         self.stt_lock = asyncio.Lock()
         self.llm_lock = asyncio.Lock()
         self.tts_lock = asyncio.Lock()
@@ -68,7 +67,7 @@ class ConversationFlow(EventEmitter[Literal["transcription"]], ABC):
         self._wait_timer: asyncio.TimerHandle | None = None
         self._transcript_processing_lock = asyncio.Lock()
 
-        # self._eou_timer_task: asyncio.Task | None = None
+        self.min_interruption_words = 1 # 2 
 
     async def start(self) -> None:
         global_event_emitter.on("speech_started", self.on_speech_started_stt)
@@ -127,6 +126,21 @@ class ConversationFlow(EventEmitter[Literal["transcription"]], ABC):
 
     async def on_stt_transcript(self, stt_response: STTResponse) -> None:
         """Handle STT transcript events with enhanced EOU logic"""
+            
+        if self._waiting_for_more_speech:
+            await self._handle_continued_speech()
+            
+        text = stt_response.data.text if stt_response.data else ""
+        
+        if self.agent.session:
+            if self.agent.session.agent_state in (AgentState.SPEAKING, AgentState.THINKING): 
+                await self.handle_stt_event(text)
+            else:
+                logger.info(f"STT TRANSCRIPT EVENT: Agent is not SPEAKING, skipping interruption, Agent state: {self.agent.session.agent_state}")
+                
+        if self.agent.session:
+            self.agent.session._emit_user_state(UserState.SPEAKING)
+            
         if stt_response.event_type == SpeechEventType.FINAL:
             user_text = stt_response.data.text
             await self._process_transcript_with_eou(user_text)
@@ -134,6 +148,8 @@ class ConversationFlow(EventEmitter[Literal["transcription"]], ABC):
     async def _process_transcript_with_eou(self, new_transcript: str) -> None:
         """Enhanced transcript processing with EOU-based decision making"""
         async with self._transcript_processing_lock:
+            if self.agent.session:
+                self.agent.session._emit_agent_state(AgentState.LISTENING) 
             # Append new transcript to accumulated transcript
             if self._accumulated_transcript:
                 self._accumulated_transcript += " " + new_transcript
@@ -209,16 +225,14 @@ class ConversationFlow(EventEmitter[Literal["transcription"]], ABC):
 
     async def _process_final_transcript(self, user_text: str) -> None:
         """Process final transcript with EOU detection and response generation"""
-
-        # Fallback: If VAD is missing, this can start the turn. Otherwise, the collector handles it.
-        if not cascading_metrics_collector.data.current_turn:
-            cascading_metrics_collector.on_user_speech_start()
+        
+        if not cascading_metrics_collector.data.current_turn: 
+            cascading_metrics_collector.start_new_interaction()
 
         cascading_metrics_collector.set_user_transcript(user_text)
         cascading_metrics_collector.on_stt_complete()
 
-        # Fallback: If VAD is present but hasn't called on_user_speech_end yet,
-        if self.vad and cascading_metrics_collector.data.is_user_speaking:
+        if self.vad and cascading_metrics_collector.data.is_user_speaking: 
             cascading_metrics_collector.on_user_speech_end()
         elif not self.vad:
             cascading_metrics_collector.on_user_speech_end()
@@ -228,10 +242,20 @@ class ConversationFlow(EventEmitter[Literal["transcription"]], ABC):
             content=user_text
         )
 
+        if self.agent and self.agent.session:
+            if self.agent.session.current_utterance and not self.agent.session.current_utterance.done():
+                self.agent.session.current_utterance.interrupt()
+            
+            handle = UtteranceHandle(utterance_id=f"utt_{uuid.uuid4().hex[:8]}")
+            self.agent.session.current_utterance = handle
+        else:
+            handle = UtteranceHandle(utterance_id="utt_fallback")
+            handle._mark_done()
+
         await self.on_turn_start(user_text)
 
         # Generate response
-        asyncio.create_task(self._generate_and_synthesize_response(user_text))
+        asyncio.create_task(self._generate_and_synthesize_response(user_text, handle))
 
         # Async helper: waits before generating a response (used if utterance isn't clearly ended)
         # async def generate_response_after_delay(delay: float):
@@ -261,7 +285,7 @@ class ConversationFlow(EventEmitter[Literal["transcription"]], ABC):
 
         await self.on_turn_end()
 
-    async def _process_reply_instructions(self, instructions: str, wait_for_playback: bool = True) -> None:
+    async def _process_reply_instructions(self, instructions: str, wait_for_playback: bool, handle: UtteranceHandle) -> None:
         """Process reply instructions and generate response using existing flow"""
         
         original_vad_handler = None
@@ -285,8 +309,7 @@ class ConversationFlow(EventEmitter[Literal["transcription"]], ABC):
             )
 
             await self.on_turn_start(instructions)
-            await self._generate_and_synthesize_response(instructions)
-
+            await self._generate_and_synthesize_response(instructions, handle)
             await self.on_turn_end()
             
             if wait_for_playback:
@@ -301,18 +324,20 @@ class ConversationFlow(EventEmitter[Literal["transcription"]], ABC):
                 
                 if original_stt_handler is not None:
                     self.on_stt_transcript = original_stt_handler
+            
+            if not handle.done():
+                handle._mark_done()
 
-    async def _generate_and_synthesize_response(self, user_text: str) -> None:
-        """Generate agent response"""
+    async def _generate_and_synthesize_response(self, user_text: str, handle: UtteranceHandle) -> None:
+        """Generate agent response and manage handle lifecycle"""
         self._is_interrupted = False
 
         full_response = ""
         self._partial_response = ""
 
         try:
-            if self.background_audio and self.tts and self.tts.audio_track:
-                self._background_audio_player = BackgroundAudio(self.background_audio, self.tts.audio_track)
-                await self._background_audio_player.start()
+            if self.agent.session and self.agent.session.is_background_audio_enabled:
+                await self.agent.session.start_thinking_audio()
 
             llm_stream = self.run(user_text)
 
@@ -322,17 +347,21 @@ class ConversationFlow(EventEmitter[Literal["transcription"]], ABC):
                 response_parts = []
                 try:
                     async for chunk in llm_stream:
-                        if self._is_interrupted:
+                        if handle.interrupted:
                             logger.info("LLM collection interrupted")
                             await q.put(None)
                             return "".join(response_parts)
 
                         self._partial_response = "".join(response_parts)
-                        await q.put(chunk)
-                        response_parts.append(chunk)
+                        
+                        if not handle.interrupted:
+                            await q.put(chunk)
+                            response_parts.append(chunk)
 
-                    await q.put(None)
-                    return "".join(response_parts)
+                    if not handle.interrupted:
+                        await q.put(None)
+                    full_response = "".join(response_parts)
+                    return full_response
                 except asyncio.CancelledError:
                     logger.info("LLM collection cancelled")
                     await q.put(None)
@@ -341,19 +370,25 @@ class ConversationFlow(EventEmitter[Literal["transcription"]], ABC):
             async def tts_consumer():
                 async def tts_stream_gen():
                     while True:
-                        if self._is_interrupted:
+                        if handle.interrupted:
                             break
 
-                        chunk = await q.get()
-                        if chunk is None:
-                            break
-                        yield chunk
+                        try:
+                            chunk = await asyncio.wait_for(q.get(), timeout=0.1)
+                            if chunk is None:
+                                break
+                            yield chunk
+                        except asyncio.TimeoutError:
+                            if handle.interrupted:
+                                break
+                            continue
 
                 if self.tts:
                     try:
                         await self._synthesize_with_tts(tts_stream_gen())
                     except asyncio.CancelledError:
-                        pass
+                        await self.tts.interrupt()
+
 
             collector_task = asyncio.create_task(collector())
             tts_task = asyncio.create_task(tts_consumer())
@@ -361,7 +396,14 @@ class ConversationFlow(EventEmitter[Literal["transcription"]], ABC):
             self._current_llm_task = collector_task
             self._current_tts_task = tts_task
 
-            await asyncio.gather(collector_task, tts_task, return_exceptions=True)
+            try:
+                await asyncio.gather(collector_task, tts_task, return_exceptions=True)
+            except asyncio.CancelledError:
+                # Ensure proper cleanup on cancellation
+                if not collector_task.done():
+                    collector_task.cancel()
+                if not tts_task.done():
+                    tts_task.cancel()
 
             if not collector_task.cancelled() and not self._is_interrupted:
                 full_response = collector_task.result()
@@ -379,6 +421,8 @@ class ConversationFlow(EventEmitter[Literal["transcription"]], ABC):
             self._current_tts_task = None
             self._current_llm_task = None
             cascading_metrics_collector.complete_current_turn()
+            if not handle.done():
+                handle._mark_done()
 
     async def process_with_llm(self) -> AsyncIterator[str]:
         """
@@ -391,6 +435,10 @@ class ConversationFlow(EventEmitter[Literal["transcription"]], ABC):
 
             cascading_metrics_collector.on_llm_start()
             first_chunk_received = False
+            
+            if self.agent.session:
+                self.agent.session._emit_user_state(UserState.IDLE)
+                self.agent.session._emit_agent_state(AgentState.THINKING)
 
             async for llm_chunk_resp in self.llm.chat(
                 self.agent.chat_context,
@@ -450,9 +498,9 @@ class ConversationFlow(EventEmitter[Literal["transcription"]], ABC):
                     if llm_chunk_resp.content:
                         yield llm_chunk_resp.content
 
-    async def say(self, message: str) -> None:
+    async def say(self, message: str, handle: UtteranceHandle) -> None:
         """
-        Direct TTS synthesis (used for initial messages)
+        Direct TTS synthesis (used for initial messages) and manage handle lifecycle.
         """
         if self.tts:
             cascading_metrics_collector.start_new_interaction("")
@@ -462,6 +510,7 @@ class ConversationFlow(EventEmitter[Literal["transcription"]], ABC):
                 await self._synthesize_with_tts(message)
             finally:
                 cascading_metrics_collector.complete_current_turn()
+                handle._mark_done()
 
     async def process_text_input(self, text: str) -> None:
         """
@@ -509,7 +558,30 @@ class ConversationFlow(EventEmitter[Literal["transcription"]], ABC):
 
     def on_speech_stopped_stt(self, event_data: Any) -> None:
         pass
-
+    
+    async def handle_stt_event(self, text: str) -> None:
+        """Handle STT event"""
+        if not text or not text.strip():
+            return
+        
+        word_count = len(text.strip().split())
+        logger.info(f"handle_stt_event: Word count: {word_count}")
+        if word_count >= self.min_interruption_words:
+            await self._trigger_interruption()
+            
+    async def _trigger_interruption(self) -> None:
+        """Trigger interruption once"""
+        logger.info(f"Triggering interruption From STT")
+        if self._is_interrupted:
+            logger.info(f"Already interrupted, skipping")
+            return
+            
+        if self.tts:
+            try:
+                await self._interrupt_tts()
+            except Exception as e:
+                logger.error(f" _trigger_interruption Callback error: {e}")
+    
     async def on_speech_started(self) -> None:       
         cascading_metrics_collector.on_user_speech_start()
 
@@ -524,17 +596,16 @@ class ConversationFlow(EventEmitter[Literal["transcription"]], ABC):
         
         if self.agent.session:
             self.agent.session._emit_user_state(UserState.SPEAKING)
-            self.agent.session._emit_agent_state(AgentState.LISTENING)
 
     async def _interrupt_tts(self) -> None:
-        logger.info("Interrupting TTS and LLM generation")
-
-        if self._background_audio_player:
-            await self._background_audio_player.stop()
-            self._background_audio_player = None
-
         self._is_interrupted = True
 
+        if self.agent and self.agent.session and self.agent.session.current_utterance:
+            self.agent.session.current_utterance.interrupt()
+
+        if self.agent.session and self.agent.session.is_background_audio_enabled:
+            await self.agent.session.stop_thinking_audio()
+
         # Cancel any waiting timers
         if self._wait_timer:
             self._wait_timer.cancel()
@@ -554,7 +625,14 @@ class ConversationFlow(EventEmitter[Literal["transcription"]], ABC):
             tasks_to_cancel.append(self._current_llm_task)
 
         if tasks_to_cancel:
-            await graceful_cancel(*tasks_to_cancel)
+            # Force cancel tasks to ensure immediate stop
+            for task in tasks_to_cancel:
+                task.cancel()
+            await asyncio.gather(*tasks_to_cancel, return_exceptions=True)
+            
+        # Reset conversation state
+        self._partial_response = ""
+        self._is_interrupted = False
 
         cascading_metrics_collector.on_interrupted()
 
@@ -574,7 +652,6 @@ class ConversationFlow(EventEmitter[Literal["transcription"]], ABC):
         
         if self.agent.session:
             self.agent.session._emit_user_state(UserState.IDLE)
-            self.agent.session._emit_agent_state(AgentState.THINKING)
 
     async def _synthesize_with_tts(self, response_gen: AsyncIterator[str] | str) -> None:
         """
@@ -582,13 +659,13 @@ class ConversationFlow(EventEmitter[Literal["transcription"]], ABC):
         """
         if not self.tts:
             return
-        
-        self.agent.session._pause_wake_up_timer()
+
+        if self.agent and self.agent.session:
+            self.agent.session._pause_wake_up_timer()
 
         async def on_first_audio_byte():
-            if self._background_audio_player:
-                await self._background_audio_player.stop()
-                self._background_audio_player = None
+            if self.agent.session and self.agent.session.is_background_audio_enabled:
+                await self.agent.session.stop_thinking_audio()
             cascading_metrics_collector.on_tts_first_byte()
             cascading_metrics_collector.on_agent_speech_start()
             
@@ -613,17 +690,19 @@ class ConversationFlow(EventEmitter[Literal["transcription"]], ABC):
             
 
         finally:
-            if self._background_audio_player:
-                await self._background_audio_player.stop()
-                self._background_audio_player = None
+            if self.agent.session and self.agent.session.is_background_audio_enabled:
+                await self.agent.session.stop_thinking_audio()
 
-            self.agent.session._reply_in_progress = False
-            self.agent.session._reset_wake_up_timer()
+            if self.agent and self.agent.session:
+                self.agent.session._reply_in_progress = False
+                self.agent.session._reset_wake_up_timer()
+            
             cascading_metrics_collector.on_agent_speech_end()
             
-            if self.agent.session:
-                self.agent.session._emit_agent_state(AgentState.IDLE)
-                self.agent.session._emit_user_state(UserState.IDLE)
+        # TODO: Need to work on IDLE state 
+            # if self.agent.session:
+            #     self.agent.session._emit_agent_state(AgentState.IDLE)
+            #     self.agent.session._emit_user_state(UserState.IDLE)
     
     async def cleanup(self) -> None:
         """Cleanup conversation flow resources"""
@@ -644,10 +723,6 @@ class ConversationFlow(EventEmitter[Literal["transcription"]], ABC):
                 pass
             self._current_llm_task = None
         
-        if self._background_audio_player:
-            await self._background_audio_player.stop()
-            self._background_audio_player = None
-
         if self._eou_timer_task and not self._eou_timer_task.done():
             self._eou_timer_task.cancel()
             try:
@@ -675,6 +750,4 @@ class ConversationFlow(EventEmitter[Literal["transcription"]], ABC):
         self._stt_started = False
         self._partial_response = ""
         self._is_interrupted = False
-        self.background_audio = None
-        self._background_audio_player = None
         logger.info("Conversation flow cleaned up")
\ No newline at end of file
